Package: CausalMixGPD
Type: Package
Title: Bayesian Nonparametric Conditional Density Modeling in Causal
        Inference and Clustering with a Heavy-Tail Extension
Version: 0.8.0
Maintainer: Arnab Aich <aaich@fsu.edu>
Authors@R: c(
    person(given = "Arnab", family = "Aich", role = c("aut", "cre"),
           email = "aaich@fsu.edu",
           comment = c(ORCID = "0009-0005-7801-6701"))
    )
Description: The presence of a heavy tail is a feature of many scenarios when risk management
            involves extremely rare events. While parametric distributions may give adequate
            representation of the mode of data, they are likely to misrepresent heavy tails,
            and completely nonparametric approaches lack a rigorous mechanism for
            tail extrapolation; see Pickands (1975) <doi:10.1214/aos/1176343003>. The
            package 'CausalMixGPD' implements tools for Bayesian analysis of heavy-tailed
            outcomes by combining Dirichlet process mixture models for the body of the
            distribution with optional generalized Pareto tails. The method allows
            for unconditional and covariate-modulated mixtures, implements MCMC estimation
            using 'nimble', and extends to mixtures of different arms' outcomes with
            application to causal inference in the Rubin (1974)
            <doi:10.1037/h0037350> framework. Posterior summaries include density
            functions, quantiles, expected values, survival functions, and causal effects,
            with an emphasis on tail quantiles and functional measures sensitive to the
            tail.
License: GPL-3
Encoding: UTF-8
LazyData: true
Depends: R (>= 4.0.0)
Imports: nimble, stats, utils, ggplot2
Suggests: testthat (>= 3.0.0), knitr, rmarkdown, here, cli, coda,
        ggmcmc, future, future.apply, crayon, DT, kableExtra, plotly,
        MatchIt, codetools
VignetteBuilder: knitr
RoxygenNote: 7.3.3
Config/testthat/edition: 3
URL: https://arnabaich96.github.io/CausalMixGPD/pkgdown/,
        https://doi.org/10.5281/zenodo.19671881
BugReports: https://arnabaich96.github.io/CausalMixGPD/
NeedsCompilation: no
Packaged: 2026-05-05 07:45:28 UTC; arnab
Author: Arnab Aich [aut, cre] (ORCID: <https://orcid.org/0009-0005-7801-6701>)
Repository: CRAN
Date/Publication: 2026-05-05 10:30:02 UTC
Built: R 4.5.2; ; 2026-05-05 11:30:50 UTC; unix
