# Generated by roxygen2: do not edit by hand

S3method(ate,causalmixgpd_causal_fit)
S3method(ate,default)
S3method(cate,causalmixgpd_causal_fit)
S3method(cate,default)
S3method(cluster_profiles,dpmixgpd_cluster_fit)
S3method(cluster_profiles,dpmixgpd_cluster_labels)
S3method(cluster_profiles,summary.dpmixgpd_cluster_fit)
S3method(cluster_profiles,summary.dpmixgpd_cluster_labels)
S3method(cqte,causalmixgpd_causal_fit)
S3method(cqte,default)
S3method(fitted,mixgpd_fit)
S3method(params,causalmixgpd_causal_fit)
S3method(params,causalmixgpd_ps_fit)
S3method(params,mixgpd_fit)
S3method(plot,causalmixgpd_ate)
S3method(plot,causalmixgpd_causal_fit)
S3method(plot,causalmixgpd_causal_predict)
S3method(plot,causalmixgpd_qte)
S3method(plot,dpmixgpd_cluster_bundle)
S3method(plot,dpmixgpd_cluster_fit)
S3method(plot,dpmixgpd_cluster_labels)
S3method(plot,dpmixgpd_cluster_psm)
S3method(plot,mixgpd_fit)
S3method(plot,mixgpd_fitted)
S3method(plot,mixgpd_predict)
S3method(predict,causalmixgpd_causal_fit)
S3method(predict,dpmixgpd_cluster_fit)
S3method(predict,mixgpd_fit)
S3method(print,causalmixgpd_ate)
S3method(print,causalmixgpd_bundle)
S3method(print,causalmixgpd_causal_bundle)
S3method(print,causalmixgpd_causal_fit)
S3method(print,causalmixgpd_causal_fit_plots)
S3method(print,causalmixgpd_causal_predict_plots)
S3method(print,causalmixgpd_ps_bundle)
S3method(print,causalmixgpd_ps_fit)
S3method(print,causalmixgpd_qte)
S3method(print,dpmixgpd_cluster_bundle)
S3method(print,dpmixgpd_cluster_fit)
S3method(print,dpmixgpd_cluster_labels)
S3method(print,dpmixgpd_cluster_psm)
S3method(print,mixgpd_ess_summary)
S3method(print,mixgpd_fit)
S3method(print,mixgpd_fit_plots)
S3method(print,mixgpd_fitted_plots)
S3method(print,mixgpd_params)
S3method(print,mixgpd_params_pair)
S3method(print,mixgpd_predict)
S3method(print,mixgpd_predict_plots)
S3method(print,mixgpd_summary)
S3method(print,summary.causalmixgpd_ate)
S3method(print,summary.causalmixgpd_causal_fit)
S3method(print,summary.causalmixgpd_ps_fit)
S3method(print,summary.causalmixgpd_qte)
S3method(qte,causalmixgpd_causal_fit)
S3method(qte,default)
S3method(residuals,mixgpd_fit)
S3method(summary,causalmixgpd_ate)
S3method(summary,causalmixgpd_bundle)
S3method(summary,causalmixgpd_causal_bundle)
S3method(summary,causalmixgpd_causal_fit)
S3method(summary,causalmixgpd_ps_bundle)
S3method(summary,causalmixgpd_ps_fit)
S3method(summary,causalmixgpd_qte)
S3method(summary,dpmixgpd_cluster_bundle)
S3method(summary,dpmixgpd_cluster_fit)
S3method(summary,dpmixgpd_cluster_labels)
S3method(summary,dpmixgpd_cluster_psm)
S3method(summary,mixgpd_ess_summary)
S3method(summary,mixgpd_fit)
export(ate)
export(ate_rmean)
export(att)
export(build_causal_bundle)
export(build_nimble_bundle)
export(bundle)
export(cate)
export(check_glue_validity)
export(cluster_profiles)
export(cqte)
export(dAmoroso)
export(dAmorosoGpd)
export(dAmorosoMix)
export(dAmorosoMixGpd)
export(dCauchy)
export(dCauchyMix)
export(dGammaGpd)
export(dGammaMix)
export(dGammaMixGpd)
export(dGpd)
export(dInvGauss)
export(dInvGaussGpd)
export(dInvGaussMix)
export(dInvGaussMixGpd)
export(dLaplaceGpd)
export(dLaplaceMix)
export(dLaplaceMixGpd)
export(dLognormalGpd)
export(dLognormalMix)
export(dLognormalMixGpd)
export(dNormGpd)
export(dNormMix)
export(dNormMixGpd)
export(damoroso)
export(damorosogpd)
export(damorosomix)
export(damorosomixgpd)
export(dcauchy_vec)
export(dcauchymix)
export(dgammagpd)
export(dgammamix)
export(dgammamixgpd)
export(dgpd)
export(dinvgauss)
export(dinvgaussgpd)
export(dinvgaussmix)
export(dinvgaussmixgpd)
export(dlaplacegpd)
export(dlaplacemix)
export(dlaplacemixgpd)
export(dlognormalgpd)
export(dlognormalmix)
export(dlognormalmixgpd)
export(dnormgpd)
export(dnormmix)
export(dnormmixgpd)
export(dpmgpd)
export(dpmgpd.causal)
export(dpmgpd.cluster)
export(dpmix)
export(dpmix.causal)
export(dpmix.cluster)
export(ess_summary)
export(get_kernel_registry)
export(get_tail_registry)
export(init_kernel_registry)
export(kernel_support_table)
export(mcmc)
export(pAmoroso)
export(pAmorosoGpd)
export(pAmorosoMix)
export(pAmorosoMixGpd)
export(pCauchy)
export(pCauchyMix)
export(pGammaGpd)
export(pGammaMix)
export(pGammaMixGpd)
export(pGpd)
export(pInvGauss)
export(pInvGaussGpd)
export(pInvGaussMix)
export(pInvGaussMixGpd)
export(pLaplaceGpd)
export(pLaplaceMix)
export(pLaplaceMixGpd)
export(pLognormalGpd)
export(pLognormalMix)
export(pLognormalMixGpd)
export(pNormGpd)
export(pNormMix)
export(pNormMixGpd)
export(pamoroso)
export(pamorosogpd)
export(pamorosomix)
export(pamorosomixgpd)
export(params)
export(pcauchy_vec)
export(pcauchymix)
export(pgammagpd)
export(pgammamix)
export(pgammamixgpd)
export(pgpd)
export(pinvgauss)
export(pinvgaussgpd)
export(pinvgaussmix)
export(pinvgaussmixgpd)
export(plaplacegpd)
export(plaplacemix)
export(plaplacemixgpd)
export(plognormalgpd)
export(plognormalmix)
export(plognormalmixgpd)
export(pnormgpd)
export(pnormmix)
export(pnormmixgpd)
export(qAmoroso)
export(qAmorosoGpd)
export(qAmorosoMix)
export(qAmorosoMixGpd)
export(qCauchy)
export(qCauchyMix)
export(qGammaGpd)
export(qGammaMix)
export(qGammaMixGpd)
export(qGpd)
export(qInvGauss)
export(qInvGaussGpd)
export(qInvGaussMix)
export(qInvGaussMixGpd)
export(qLaplaceGpd)
export(qLaplaceMix)
export(qLaplaceMixGpd)
export(qLognormalGpd)
export(qLognormalMix)
export(qLognormalMixGpd)
export(qNormGpd)
export(qNormMix)
export(qNormMixGpd)
export(qamoroso)
export(qamorosogpd)
export(qamorosomix)
export(qamorosomixgpd)
export(qcauchy_vec)
export(qcauchymix)
export(qgammagpd)
export(qgammamix)
export(qgammamixgpd)
export(qgpd)
export(qinvgauss)
export(qinvgaussgpd)
export(qinvgaussmix)
export(qinvgaussmixgpd)
export(qlaplacegpd)
export(qlaplacemix)
export(qlaplacemixgpd)
export(qlognormalgpd)
export(qlognormalmix)
export(qlognormalmixgpd)
export(qnormgpd)
export(qnormmix)
export(qnormmixgpd)
export(qte)
export(qtt)
export(rAmoroso)
export(rAmorosoGpd)
export(rAmorosoMix)
export(rAmorosoMixGpd)
export(rCauchy)
export(rCauchyMix)
export(rGammaGpd)
export(rGammaMix)
export(rGammaMixGpd)
export(rGpd)
export(rInvGauss)
export(rInvGaussGpd)
export(rInvGaussMix)
export(rInvGaussMixGpd)
export(rLaplaceGpd)
export(rLaplaceMix)
export(rLaplaceMixGpd)
export(rLognormalGpd)
export(rLognormalMix)
export(rLognormalMixGpd)
export(rNormGpd)
export(rNormMix)
export(rNormMixGpd)
export(ramoroso)
export(ramorosogpd)
export(ramorosomix)
export(ramorosomixgpd)
export(rcauchy_vec)
export(rcauchymix)
export(rgammagpd)
export(rgammamix)
export(rgammamixgpd)
export(rgpd)
export(rinvgauss)
export(rinvgaussgpd)
export(rinvgaussmix)
export(rinvgaussmixgpd)
export(rlaplacegpd)
export(rlaplacemix)
export(rlaplacemixgpd)
export(rlognormalgpd)
export(rlognormalmix)
export(rlognormalmixgpd)
export(rnormgpd)
export(rnormmix)
export(rnormmixgpd)
export(run_mcmc_bundle_manual)
export(run_mcmc_causal)
export(sim_bulk_tail)
export(sim_causal_qte)
export(sim_survival_tail)
import(ggplot2)
import(nimble)
importFrom(nimble,ddexp)
importFrom(nimble,nimNumeric)
importFrom(nimble,pdexp)
importFrom(nimble,qdexp)
importFrom(nimble,rdexp)
importFrom(stats,coef)
importFrom(stats,dcauchy)
importFrom(stats,density)
importFrom(stats,dgamma)
importFrom(stats,dlnorm)
importFrom(stats,dnorm)
importFrom(stats,fitted)
importFrom(stats,pcauchy)
importFrom(stats,pgamma)
importFrom(stats,plnorm)
importFrom(stats,plogis)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,qcauchy)
importFrom(stats,qgamma)
importFrom(stats,qlnorm)
importFrom(stats,qlogis)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rbinom)
importFrom(stats,rcauchy)
importFrom(stats,residuals)
importFrom(stats,rexp)
importFrom(stats,rgamma)
importFrom(stats,rlnorm)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,uniroot)
importFrom(utils,capture.output)
importFrom(utils,head)
