% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/internal.R
\name{.wrap_scalar_r}
\alias{.wrap_scalar_r}
\title{Wrap scalar RNG to handle n > 1.}
\usage{
.wrap_scalar_r(fun)
}
\description{
Wrap scalar RNG to handle n > 1.
}
\details{
Random-generation helpers in the package are scalar-at-a-time. This wrapper
promotes them to the standard \code{n} interface by repeating the scalar generator
and returning either a numeric vector or a matrix of generated values,
depending on the length of one draw.
}
\keyword{internal}
