% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cluster.R
\name{cluster_profiles}
\alias{cluster_profiles}
\title{Extract Cluster Profiles}
\usage{
cluster_profiles(object, ...)
}
\arguments{
\item{object}{A cluster fit, cluster-label object, or corresponding summary object.}

\item{...}{Additional arguments passed to summary methods for fitted or label objects.}
}
\value{
A data frame of cluster-level descriptive summaries, or \code{NULL}
when profiles are unavailable.
}
\description{
Access descriptive cluster profiles without reaching into summary internals.
}
\seealso{
Other cluster workflow: 
\code{\link{dpmgpd.cluster}()},
\code{\link{dpmix.cluster}()},
\code{\link{plot.dpmixgpd_cluster_bundle}()},
\code{\link{predict.dpmixgpd_cluster_fit}()},
\code{\link{print.dpmixgpd_cluster_bundle}()},
\code{\link{print.dpmixgpd_cluster_fit}()},
\code{\link{print.dpmixgpd_cluster_labels}()},
\code{\link{print.dpmixgpd_cluster_psm}()},
\code{\link{summary.dpmixgpd_cluster_bundle}()},
\code{\link{summary.dpmixgpd_cluster_fit}()},
\code{\link{summary.dpmixgpd_cluster_labels}()},
\code{\link{summary.dpmixgpd_cluster_psm}()}
}
\concept{cluster workflow}
